import redis
from django.utils.encoding import force_unicode
from django.contrib.sessions.backends.base import SessionBase, CreateError
from django.conf import settings


class SessionStore(SessionBase):
    """
    Implements Redis database session store.
    """
    def __init__(self, session_key=None):
        super(SessionStore, self).__init__(session_key)
        self.server = redis.StrictRedis(
            host=getattr(settings, 'SESSION_REDIS_HOST', 'localhost'),
            port=getattr(settings, 'SESSION_REDIS_PORT', 6379),
            db=getattr(settings, 'SESSION_REDIS_DB', 0),
            password=getattr(settings, 'SESSION_REDIS_PASSWORD', None)
        )

    def load(self):
        try:
            session_data = self.server.get(self.get_real_stored_key(self.session_key))
            return self.decode(force_unicode(session_data))
        except:
            self.create()
            return {}

    def exists(self, session_key):
        return self.server.exists(self.get_real_stored_key(session_key))

    def create(self):
        while True:
            self._session_key = self._get_new_session_key()
            
            try:
                self.save(must_create=True)
            except CreateError:
                continue
            self.modified = True
            return

    def save(self, must_create=False):
        if must_create and self.exists(self.session_key):
            raise CreateError
        data = self.encode(self._get_session(no_load=must_create))
        self.server.set(self.get_real_stored_key(self.session_key), data)
        self.server.expire(self.get_real_stored_key(self.session_key), self.get_expiry_age())

    def delete(self, session_key=None):
        if session_key is None:
            if self._session_key is None:
                return
            session_key = self._session_key
        try:
            self.server.delete(self.get_real_stored_key(session_key))
        except:
            pass

    def get_real_stored_key(self, session_key):
        """Return the real key name in redis storage
        @return string
        """
        prefix = getattr(settings, 'SESSION_REDIS_PREFIX', '')
        if not prefix:
            return session_key
        return ':'.join([prefix, session_key])



from session import SessionStore as RedisSession
import unittest
import time
from nose.tools import eq_


class TestRedisSesssios(unittest.TestCase):

    def setUp(self):
        self.redis_session = RedisSession()

    def test_modify_and_keys(self):
        eq_(self.redis_session.modified, False)
        self.redis_session['test'] = 'test_me'
        eq_(self.redis_session.modified, True)
        eq_(self.redis_session['test'], 'test_me')

    def test_save_and_delete(self):
        self.redis_session['key'] = 'value'
        self.redis_session.save()
        eq_(self.redis_session.exists(self.redis_session.session_key), True)
        self.redis_session.delete(self.redis_session.session_key)
        eq_(self.redis_session.exists(self.redis_session.session_key), False)

    def test_flush(self):
        self.redis_session['key'] = 'another_value'
        self.redis_session.save()
        key = self.redis_session.session_key
        self.redis_session.flush()
        eq_(self.redis_session.exists(key), False)

    def test_items(self):
        self.redis_session['item1'], self.redis_session['item2'] = 1, 2
        self.redis_session.save()
        eq_(self.redis_session.items(), [('item2', 2), ('item1', 1)])

    def test_expiry(self):
        self.redis_session.set_expiry(1)
        # Test if the expiry age is set correctly
        eq_(self.redis_session.get_expiry_age(), 1)
        self.redis_session['key'] = 'expiring_value'
        self.redis_session.save()
        key = self.redis_session.session_key
        eq_(self.redis_session.exists(key), True)
        time.sleep(2)
        eq_(self.redis_session.exists(key), False)

    def test_save_and_load(self):
        self.redis_session.set_expiry(60)
        self.redis_session.setdefault('item_test', 8)
        self.redis_session.save()
        session_data = self.redis_session.load()
        eq_(session_data.get('item_test'), 8)

    #def test_load(self):
        #self.redis_session.set_expiry(60)
        #self.redis_session['item1'], self.redis_session['item2'] = 1,2
        #self.redis_session.save()
        #session_data = self.redis_session.server.get(self.redis_session.session_key)
        #expiry, data = int(session_data[:15]), session_data[15:]

if __name__ == '__main__':
    import os
    os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'
    unittest.main()

# To run the test suite
# pip install nose
# python tests.py -v
